use ockam::{
    identity::{Identity, SecureChannelListenerOptions},
    node, Context, Result, TcpListenerOptions, TcpTransportExtension,
};

use clap::{Parser, Subcommand};

use nabi::{
    packet::{NabiPacket, User},
    receive::Receiver,
    transmit::Sender,
};

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
struct Cli {
    /// host for your TCP connection
    host: String,
}

#[ockam::node]
async fn main(ctx: Context) -> Result<()> {
    let args = Cli::parse();

    // Create a node with default implementations
    let mut node = node(ctx);

    // initialize TCP
    let tcp = node.create_tcp_transport().await?;

    // Start a worker, of type Receiver, at address "receiver"
    node.start_worker("", Sender).await?;
    let user_identity = node.create_identity().await?;
    let user = User::new(&user_identity, "USER ALIAS".to_string())?;

    // create TCP listener
    let listener = tcp.listen(args.host, TcpListenerOptions::new()).await?;

    // ockam channel listener
    let secure_channel_listener = node
        .create_secure_channel_listener(
            &user_identity,
            "",
            SecureChannelListenerOptions::new().as_consumer(listener.flow_control_id()),
        )
        .await?;
    node.flow_controls().add_consumer("re", secure_channel_listener.flow_control_id());

    // Send a message to the worker at address "receiver".
    let initial_msg = NabiPacket::new("HELLO WORLD".to_string(), user, 200)?;
    node.send("", initial_msg).await?;

    // Wait to receive a reply and print it.
    let reply = node.receive::<NabiPacket>().await?;
    println!("{}", reply);

    node.stop().await
}
