use std::fmt::Display;

use chrono;
use ockam::{self, identity::IdentityIdentifier, Message, Result};
use serde::{Deserialize, Serialize};

use pt;

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct User {
    identity: IdentityIdentifier,
    alias: String,
}

#[derive(Debug, Clone, Deserialize, Serialize)]
pub struct NabiPacket {
    text: String,
    timestamp: chrono::DateTime<chrono::Utc>,
    author: User,
    code: u16,
}

impl Message for NabiPacket {
    // pub trait Message: Encodable + Decodable + Send + 'static { }
}

impl Display for NabiPacket {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}\n\
            Author: {}\n\
            Time:   {}\n\
            \n\
            {}\n\
            ",
            pt::common::printing::divider!(),
            self.author.alias,
            self.timestamp,
            self.text,
        )
    }
}

impl User {
    pub fn new(identity: &IdentityIdentifier, alias: String) -> Result<Self> {
        let user = User {
            identity: identity.clone(),
            alias,
        };
        Ok(user)
    }
}

impl NabiPacket {
    pub fn new(text: String, author: User, code: u16) -> Result<Self> {
        let msg = NabiPacket {
            text,
            timestamp: chrono::Utc::now(),
            author,
            code,
        };
        Ok(msg)
    }
}
